 /*
  * sigact.c
  * Copyright (C) Aitor Cuadrado Zubizarreta <aitor_czr@gnuinos.org>
  * 
  * simple-netaid is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  * 
  * simple-netaid is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License along
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * See the COPYING file.
  */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <unistd.h>
#include <ftw.h>          /*  nftw  */
#include <sys/un.h>      /*  UNIX socket  */
#include <sys/types.h>  
#include <sys/socket.h>
#include <sys/wait.h>
#include <assert.h>
#include <inttypes.h>
#include <ctype.h>       /* ???   */

#include <simple-netaid/sbuf.h>
#include <simple-netaid/helpers.h>
#include <simple-netaid/interfaces.h>
#include <simple-netaid/ipaddr.h>
#include <simple-netaid/wireless.h>
#include <simple-netaid/iproute.h>
#include <simple-netaid/ethlink.h>

#include "sigact.h"
#include "def.h"
#include "config.h"
#include "util.h"
#include "periodic_reminder.h"

#define MAX_SIZE 1024

/* This flag controls termination of the main loop. */
static volatile sig_atomic_t keep_going = 1;
static int count = 0; /* counter */

static
void wake_up()
{
   sbuf_t wired __cleanbuf__;
   sbuf_t wireless __cleanbuf__;
   
   keep_going = 0;
   
   sbuf_init(&wired);
   sbuf_init(&wireless);
   get_interfaces(&wired, &wireless);
   
   if (wired.len>2) {   
      
      //if (!get_interface_status(wired.buf))
      interface_up(wired.buf);
      if (ethlink(wired.buf) && get_interface_status(wired.buf))
         wired_connection(wired.buf);   
   }  
   
   sbuf_free(&wired);
   
   if (wireless.len>2 && *iproute() == 0) {
      
      //if (!get_interface_status(wireless.buf))
      interface_up(wireless.buf);
      if (get_interface_status(wireless.buf))
	     wireless_connection(wireless.buf);
   }
 
   sbuf_free(&wireless);

   keep_going = 1;
}

void sig_handler(int signum, siginfo_t *info, void *extra)
{ 
   switch(signum) {
      
      case SIGALRM:
         if(keep_going) {
            if(*iproute() == 0)
               wake_up();
         }      
         break;
        
      case SIGHUP:
         break;
        
      case SIGUSR1:
         break;
         
      case SIGTERM:
      case SIGINT:
      case SIGQUIT:
         {                
            stop_periodic();

            // Ubus loop part
            if( access( m_config->pidfile_path, F_OK ) == 0 ) {
               uloop_done();
               ubus_free(ctx);
               if(pid_fd != -1) {
                  int rc = lockf(pid_fd, F_ULOCK, 0);
                  if(rc != 0) exit(EXIT_FAILURE);
                  close(pid_fd);
               }
               unlink(m_config->pidfile_path);
            }
         }
         exit(EXIT_SUCCESS);  /* exit gracefully. */
         break;
         
      default:
         break;
   }
}

void sigaction_init(void)
{
   struct sigaction sa;
   memset (&sa, 0, sizeof(sa));
      
   /*---------------- initialize signal handling ------------ */
   sa.sa_sigaction = &sig_handler;
   sa.sa_flags = SA_RESTART | SA_NODEFER | SA_SIGINFO; //  SA_RESETHAND | 
   sigemptyset (&sa.sa_mask);
   sigaction (SIGALRM, &sa, 0);
   sigaction (SIGCHLD, &sa, 0);
   sigaction (SIGHUP,  &sa, 0);
   sigaction (SIGUSR1, &sa, 0);
   sigaction (SIGUSR2, &sa, 0);
   sigaction (SIGTERM, &sa, 0);
   sigaction (SIGINT,  &sa, 0);
   sigaction (SIGQUIT, &sa, 0);
}
